#include "tqtglobaldefines.h"

#if 0

/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the Qt3Support module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef TQACCEL_H
#define TQACCEL_H

#include <QtCore/qobject.h>
#include <QtGui/qkeysequence.h>

QT_BEGIN_HEADER

QT_BEGIN_NAMESPACE

QT_MODULE(Qt3SupportLight)

class TQAccelPrivate;

class Q_COMPAT_EXPORT TQAccel : public TQT_BASE_OBJECT_NAME			// accelerator class
{
    Q_OBJECT
public:
    TQAccel( QWidget *parent, const char *name=0 );
    TQAccel( QWidget* watch, TQT_BASE_OBJECT_NAME *parent, const char *name=0 );
    ~TQAccel();

    bool isEnabled() const;
    void setEnabled( bool );

    uint count() const;

    int insertItem( const QKeySequence& key, int id=-1);
    void removeItem( int id );
    void clear();

    QKeySequence key( int id );
    int findKey( const QKeySequence& key ) const;

    bool isItemEnabled( int id ) const;
    void setItemEnabled( int id, bool enable );

    bool connectItem( int id, const TQT_BASE_OBJECT_NAME *receiver, const char* member );
    bool disconnectItem( int id, const TQT_BASE_OBJECT_NAME *receiver, const char* member );

    void repairEventFilter() {}

    void setWhatsThis( int id, const QString& );
    QString whatsThis( int id ) const;
    void setIgnoreWhatsThis( bool );
    bool ignoreWhatsThis() const;

    static QKeySequence shortcutKey( const QString & );
    static QString keyToString(QKeySequence k );
    static QKeySequence stringToKey( const QString & );

Q_SIGNALS:
    void activated( int id );
    void activatedAmbiguously( int id );

private:
    TQAccelPrivate * d;

private:
    Q_DISABLE_COPY(TQAccel)
    friend class TQAccelPrivate;
    friend class TQAccelManager;
};

QT_END_NAMESPACE

QT_END_HEADER

#endif // TQACCEL_H

#else // USE_QT4

/****************************************************************************
**
** Definition of TQAccel class
**
** Created : 950419
**
** Copyright (C) 2010 Timothy Pearson and (C) 1992-2008 Trolltech ASA.
**
** This file is part of the kernel module of the TQt GUI Toolkit.
**
** This file may be used under the terms of the GNU General
** Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the files LICENSE.GPL2
** and LICENSE.GPL3 included in the packaging of this file.
** Alternatively you may (at your option) use any later version
** of the GNU General Public License if such license has been
** publicly approved by Trolltech ASA (or its successors, if any)
** and the KDE Free TQt Foundation.
**
** Please review the following information to ensure GNU General
** Public Licensing requirements will be met:
** http://trolltech.com/products/qt/licenses/licensing/opensource/.
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
** or contact the sales department at sales@trolltech.com.
**
** This file may be used under the terms of the Q Public License as
** defined by Trolltech ASA and appearing in the file LICENSE.TQPL
** included in the packaging of this file.  Licensees holding valid TQt
** Commercial licenses may use this file in accordance with the TQt
** Commercial License Agreement provided with the Software.
**
** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
** herein.
**
**********************************************************************/

#ifndef TQACCEL_H
#define TQACCEL_H

#ifndef TQT_H
#include "tqobject.h"
#include "tqkeysequence.h"
#include "tqapplication.h"
#endif // TQT_H

#ifndef TQT_NO_ACCEL

class TQAccelPrivate;

class TQ_EXPORT TQAccel : public TQObject			// accelerator class
{
    Q_OBJECT
    TQ_OBJECT
public:
    TQAccel( TQWidget *parent, const char *name=0 );
    TQAccel( TQWidget* watch, TQObject *parent, const char *name=0 );
    TQAccel( TQWidget* watch, TQWidget *parent, const char *name=0 );
    ~TQAccel();

    bool isEnabled() const;
    void setEnabled( bool );

    uint count() const;

    int insertItem( const TQKeySequence& key, int id=-1);
    void removeItem( int id );
    void clear();

    TQKeySequence key( int id );
    int findKey( const TQKeySequence& key ) const;

    bool isItemEnabled( int id ) const;
    void setItemEnabled( int id, bool enable );

    bool connectItem( int id,  const TQT_BASE_OBJECT_NAME *receiver, const char* member );
    bool disconnectItem( int id,  const TQT_BASE_OBJECT_NAME *receiver, const char* member );

    void repairEventFilter();

    void setWhatsThis( int id, const TQString& );
    TQString whatsThis( int id ) const;
    void setIgnoreWhatsThis( bool );
    bool ignoreWhatsThis() const;

    static TQKeySequence shortcutKey( const TQString & );
    static TQString keyToString(TQKeySequence k );
    static TQKeySequence stringToKey( const TQString & );

Q_SIGNALS:
    void activated( int id );
    void activatedAmbiguously( int id );

protected:
    bool eventFilter( TQObject *, TQEvent * );

private:
    TQAccelPrivate * d;

private:
#if defined(TQ_DISABLE_COPY)
    TQAccel( const TQAccel & );
    TQAccel &operator=( const TQAccel & );
#endif
    friend class TQAccelPrivate;
    friend class TQAccelManager;

public:
    static bool sendSpontaneousKeyEvent( TQObject *receiver, TQKeyEvent *event );
};

inline bool TQAccel::sendSpontaneousKeyEvent( TQObject *receiver, TQKeyEvent *event )
{ return tqApp ? tqApp->notify( receiver, event ) : FALSE; }

#endif // TQT_NO_ACCEL
#endif // TQACCEL_H

#endif // USE_QT4