/****************************************************************************
**
** Definition of TQBrush class
**
** Created : 940112
**
** Copyright (C) 2010 Timothy Pearson and (C) 1992-2008 Trolltech ASA.
**
** This file is part of the kernel module of the TQt GUI Toolkit.
**
** This file may be used under the terms of the GNU General
** Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the files LICENSE.GPL2
** and LICENSE.GPL3 included in the packaging of this file.
** Alternatively you may (at your option) use any later version
** of the GNU General Public License if such license has been
** publicly approved by Trolltech ASA (or its successors, if any)
** and the KDE Free TQt Foundation.
**
** Please review the following information to ensure GNU General
** Public Licensing requirements will be met:
** http://trolltech.com/products/qt/licenses/licensing/opensource/.
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
** or contact the sales department at sales@trolltech.com.
**
** This file may be used under the terms of the Q Public License as
** defined by Trolltech ASA and appearing in the file LICENSE.TQPL
** included in the packaging of this file.  Licensees holding valid TQt
** Commercial licenses may use this file in accordance with the TQt
** Commercial License Agreement provided with the Software.
**
** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
** herein.
**
**********************************************************************/

#ifndef TQBRUSH_H
#define TQBRUSH_H

#include "tqtglobaldefines.h"

#ifndef TQT_H
#include "tqcolor.h"
#include "tqshared.h"
#endif // TQT_H

#ifdef USE_QT4

#include "tqpixmap.h"
#include <Qt/qbrush.h>

#endif // USE_QT4

#ifdef USE_QT4

class TQ_EXPORT TQBrush : public QBrush, virtual public TQt
{
public:
	TQBrush() : QBrush() {}
	TQBrush( Qt::BrushStyle bs ) : QBrush( bs ) {}
	TQBrush( const QColor &qc, Qt::BrushStyle bs=Qt::SolidPattern ) : QBrush( qc, bs ) {}
	TQBrush( const QColor &qc, TQt::BrushStyle bs ) : QBrush( qc, (Qt::BrushStyle)bs ) {}
	TQBrush( const QColor &qc, const QPixmap &qp ) : QBrush( qc, qp ) {}
	TQBrush( const QBrush &qb ) : QBrush( qb ) {}

	TQBrush(Qt::GlobalColor color, Qt::BrushStyle bs=Qt::SolidPattern) : QBrush( color, bs ) {}
	TQBrush(Qt::GlobalColor color, TQt::BrushStyle bs) : QBrush( color, (Qt::BrushStyle)bs ) {}
	TQBrush(Qt::GlobalColor color, const QPixmap &pixmap) : QBrush( color, pixmap ) {}

	TQBrush(const QPixmap &pixmap) : QBrush( pixmap ) {}
	TQBrush(const QImage &image) : QBrush( image ) {}
	TQBrush(const QGradient &gradient) : QBrush( gradient ) {}

	inline operator const QColor&() const;
	TQPixmap *pixmap() const { return const_cast<TQPixmap*>(&TQT_TQPIXMAP_OBJECT(texture())); }
	inline void setPixmap(const QPixmap &pixmap) { setTexture(pixmap); }

	// Interoperability
	static const TQBrush& convertFromQBrush( QBrush& qbr );

	using QBrush::operator=;
};

// Interoperability
inline static const TQBrush& convertFromQBrush( const QBrush& qbr ) {
	return (*static_cast<const TQBrush*>(&qbr));
}

#else // USE_QT4

class TQ_EXPORT TQBrush: public TQt
{
friend class TQPainter;
public:
    TQBrush();
    TQBrush( BrushStyle );
    TQBrush( const TQColor &, BrushStyle=SolidPattern );
    TQBrush( const TQColor &, const TQPixmap & );
    TQBrush( const TQBrush & );
   ~TQBrush();
    TQBrush &operator=( const TQBrush & );

    BrushStyle	style()	 const		{ return data->style; }
    void	setStyle( BrushStyle );
    const TQColor &color()const		{ return data->color; }
    void	setColor( const TQColor & );
    TQPixmap    *pixmap() const		{ return data->pixmap; }
    void	setPixmap( const TQPixmap & );

    bool	operator==( const TQBrush &p ) const;
    bool	operator!=( const TQBrush &b ) const
					{ return !(operator==(b)); }

private:
    TQBrush	copy()	const;
    void	detach();
    void	init( const TQColor &, BrushStyle );
    struct TQBrushData : public TQShared {	// brush data
	BrushStyle style;
	TQColor	  color;
	TQPixmap	 *pixmap;
    } *data;
};


/*****************************************************************************
  TQBrush stream functions
 *****************************************************************************/

#ifndef TQT_NO_DATASTREAM
TQ_EXPORT TQDataStream &operator<<( TQDataStream &, const TQBrush & );
TQ_EXPORT TQDataStream &operator>>( TQDataStream &, TQBrush & );
#endif

#endif // USE_QT4

#endif // TQBRUSH_H
