#include "tqtglobaldefines.h"

#ifdef USE_QT4

/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the Qt3Support module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef TQDNS_H
#define TQDNS_H

#include <Qt/qobject.h>
#include <Qt/qhostaddress.h>
#include <Qt/qsocketnotifier.h>
#include <tqstringlist.h>
#include <tqvaluelist.h>

#include "tqtimer.h"
#include "tqobject.h"
#include "tqhostaddress.h"
#include "tqsocketnotifier.h"

#define TQLatin1String TQString
#define TQLatin1Char TQChar

typedef qint8 Q_INT8;
typedef quint8 Q_UINT8;
typedef qint16 Q_INT16;
typedef quint16 Q_UINT16;
typedef qint32 Q_INT32;
typedef quint32 Q_UINT32;
typedef qint64 Q_INT64;
typedef quint64 Q_UINT64;
typedef qint64 Q_LLONG;
typedef quint64 Q_ULLONG;
#if defined(Q_OS_WIN64)
typedef __int64 Q_LONG;             /* word up to 64 bit signed */
typedef unsigned __int64 Q_ULONG;   /* word up to 64 bit unsigned */
#else
typedef long Q_LONG;                /* word up to 64 bit signed */
typedef unsigned long Q_ULONG;      /* word up to 64 bit unsigned */
#endif

QT_BEGIN_HEADER

QT_BEGIN_NAMESPACE

QT_MODULE(Qt3Support)

#ifndef QT_NO_DNS

//#define Q_DNS_SYNCHRONOUS

class TQDnsPrivate;

class Q_COMPAT_EXPORT TQDns: public TQObject {
    Q_OBJECT
    TQ_OBJECT
public:
    enum RecordType {
	None,
	A, Aaaa,
	Mx, Srv,
	Cname,
	Ptr,
	Txt
    };

    TQDns();
    TQDns( const TQString & label, RecordType rr = A );
    TQDns( const TQHostAddress & address, RecordType rr = Ptr );
    virtual ~TQDns();

    // to set/change the query
    virtual void setLabel( const TQString & label );
    virtual void setLabel( const TQHostAddress & address );
    TQString label() const { return l; }

    virtual void setRecordType( RecordType rr = A );
    RecordType recordType() const { return t; }

    // whether something is happening behind the scenes
    bool isWorking() const;

    // to query for replies
    TQValueList<TQHostAddress> addresses() const;

    class Q_COMPAT_EXPORT MailServer {
    public:
	MailServer( const TQString & n=TQString(), Q_UINT16 p=0 )
	    :name(n), priority(p) {}
	TQString name;
	Q_UINT16 priority;
	Q_DUMMY_COMPARISON_OPERATOR(MailServer)
    };
    TQValueList<MailServer> mailServers() const;

    class Q_COMPAT_EXPORT Server {
    public:
	Server(const TQString & n=TQString(), Q_UINT16 p=0, Q_UINT16 w=0, Q_UINT16 po=0 )
	    : name(n), priority(p), weight(w), port(po) {}
	TQString name;
	Q_UINT16 priority;
	Q_UINT16 weight;
	Q_UINT16 port;
	Q_DUMMY_COMPARISON_OPERATOR(Server)
    };
    TQValueList<Server> servers() const;

    TQStringList hostNames() const;

    TQStringList texts() const;

    TQString canonicalName() const; // ### real-world but uncommon: TQStringList

    TQStringList qualifiedNames() const { return n; }

#if defined(Q_DNS_SYNCHRONOUS)
protected:
    void connectNotify( const char *signal );
#endif

Q_SIGNALS:
    void resultsReady();

private Q_SLOTS:
    void startQuery();

private:
    static void doResInit();
    void setStartQueryTimer();
    static TQString toInAddrArpaDomain( const TQHostAddress &address );
#if defined(Q_DNS_SYNCHRONOUS)
    void doSynchronousLookup();
#endif

    TQString l;
    TQStringList n;
    RecordType t;
    TQDnsPrivate * d;

    friend class TQDnsAnswer;
    friend class TQDnsManager;
};


// TQDnsSocket are sockets that are used for DNS lookup

class TQDnsSocket: public TQObject {
    Q_OBJECT
    // note: Private not public.  This class contains NO public API.
protected:
    TQDnsSocket( TQT_BASE_OBJECT_NAME *, const char * );
    virtual ~TQDnsSocket();

private Q_SLOTS:
    virtual void cleanCache();
    virtual void retransmit();
    virtual void answer();
};

#endif // QT_NO_DNS

QT_END_NAMESPACE

QT_END_HEADER

#endif // TQDNS_H

#else // USE_QT4

/****************************************************************************
**
** Definition of TQDns class.
**
** Created : 991122
**
** Copyright (C) 1999-2008 Trolltech ASA.  All rights reserved.
**
** This file is part of the network module of the TQt GUI Toolkit.
**
** This file may be used under the terms of the GNU General
** Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the files LICENSE.GPL2
** and LICENSE.GPL3 included in the packaging of this file.
** Alternatively you may (at your option) use any later version
** of the GNU General Public License if such license has been
** publicly approved by Trolltech ASA (or its successors, if any)
** and the KDE Free TQt Foundation.
**
** Please review the following information to ensure GNU General
** Public Licensing requirements will be met:
** http://trolltech.com/products/qt/licenses/licensing/opensource/.
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
** or contact the sales department at sales@trolltech.com.
**
** This file may be used under the terms of the Q Public License as
** defined by Trolltech ASA and appearing in the file LICENSE.TQPL
** included in the packaging of this file.  Licensees holding valid TQt
** Commercial licenses may use this file in accordance with the TQt
** Commercial License Agreement provided with the Software.
**
** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
** herein.
**
**********************************************************************/

#ifndef TQDNS_H
#define TQDNS_H

#ifndef TQT_H
#include "tqobject.h"
#include "tqhostaddress.h"
#include "tqsocketnotifier.h"
#include "tqstringlist.h"
#endif // TQT_H

#if !defined( TQT_MODULE_NETWORK ) || defined( TQT_LICENSE_PROFESSIONAL ) || defined( TQT_INTERNAL_NETWORK )
#define TQM_EXPORT_DNS
#else
#define TQM_EXPORT_DNS TQ_EXPORT
#endif

#ifndef TQT_NO_DNS

//#define TQ_DNS_SYNCHRONOUS

class TQDnsPrivate;

class TQM_EXPORT_DNS TQDns: public TQObject {
    Q_OBJECT
    TQ_OBJECT
public:
    enum RecordType {
	None,
	A, Aaaa,
	Mx, Srv,
	Cname,
	Ptr,
	Txt
    };

    TQDns();
    TQDns( const TQString & label, RecordType rr = A );
    TQDns( const TQHostAddress & address, RecordType rr = Ptr );
    virtual ~TQDns();

    // to set/change the query
    virtual void setLabel( const TQString & label );
    virtual void setLabel( const TQHostAddress & address );
    TQString label() const { return l; }

    virtual void setRecordType( RecordType rr = A );
    RecordType recordType() const { return t; }

    // whether something is happening behind the scenes
    bool isWorking() const;

    // to query for replies
    TQValueList<TQHostAddress> addresses() const;

    class TQM_EXPORT_DNS MailServer {
    public:
	MailServer( const TQString & n=TQString::null, TQ_UINT16 p=0 )
	    :name(n), priority(p) {}
	TQString name;
	TQ_UINT16 priority;
	TQ_DUMMY_COMPARISON_OPERATOR(MailServer)
    };
    TQValueList<MailServer> mailServers() const;

    class TQM_EXPORT_DNS Server {
    public:
	Server(const TQString & n=TQString::null, TQ_UINT16 p=0, TQ_UINT16 w=0, TQ_UINT16 po=0 )
	    : name(n), priority(p), weight(w), port(po) {}
	TQString name;
	TQ_UINT16 priority;
	TQ_UINT16 weight;
	TQ_UINT16 port;
	TQ_DUMMY_COMPARISON_OPERATOR(Server)
    };
    TQValueList<Server> servers() const;

    TQStringList hostNames() const;

    TQStringList texts() const;

    TQString canonicalName() const; // ### real-world but uncommon: TQStringList

    TQStringList qualifiedNames() const { return n; }

#if defined(TQ_DNS_SYNCHRONOUS)
protected:
    void connectNotify( const char *signal );
#endif

Q_SIGNALS:
    void resultsReady();

private Q_SLOTS:
    void startQuery();

private:
    static void doResInit();
    void setStartQueryTimer();
    static TQString toInAddrArpaDomain( const TQHostAddress &address );
#if defined(TQ_DNS_SYNCHRONOUS)
    void doSynchronousLookup();
#endif

    TQString l;
    TQStringList n;
    RecordType t;
    TQDnsPrivate * d;

    friend class TQDnsAnswer;
    friend class TQDnsManager;
};


// TQDnsSocket are sockets that are used for DNS lookup

class TQDnsSocket: public TQObject {
    Q_OBJECT
    TQ_OBJECT
    // note: Private not public.  This class contains NO public API.
protected:
    TQDnsSocket( TQObject *, const char * );
    virtual ~TQDnsSocket();

private Q_SLOTS:
    virtual void cleanCache();
    virtual void retransmit();
    virtual void answer();
};

#endif // TQT_NO_DNS

#endif // TQDNS_H

#endif // USE_QT4