/****************************************************************************
**
** Definition of TQObject class
**
** Created : 930418
**
** Copyright (C) 2010 Timothy Pearson and (C) 1992-2008 Trolltech ASA.
**
** This file is part of the kernel module of the TQt GUI Toolkit.
**
** This file may be used under the terms of the GNU General
** Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the files LICENSE.GPL2
** and LICENSE.GPL3 included in the packaging of this file.
** Alternatively you may (at your option) use any later version
** of the GNU General Public License if such license has been
** publicly approved by Trolltech ASA (or its successors, if any)
** and the KDE Free TQt Foundation.
**
** Please review the following information to ensure GNU General
** Public Licensing requirements will be met:
** http://trolltech.com/products/qt/licenses/licensing/opensource/.
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
** or contact the sales department at sales@trolltech.com.
**
** This file may be used under the terms of the Q Public License as
** defined by Trolltech ASA and appearing in the file LICENSE.TQPL
** included in the packaging of this file.  Licensees holding valid TQt
** Commercial licenses may use this file in accordance with the TQt
** Commercial License Agreement provided with the Software.
**
** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
** herein.
**
**********************************************************************/

#ifndef TQOBJECT_H
#define TQOBJECT_H

#include "tqtglobaldefines.h"

#ifndef TQT_H
#include "tqobjectdefs.h"
#include "tqwindowdefs.h"
#include "tqstring.h"
#include "tqevent.h"
#include "tqnamespace.h"
#include "tqobjectlist.h"
#endif // TQT_H

#define TQT_TR_NOOP(x) (x)
#define TQT_TRANSLATE_NOOP(scope,x) (x)

class TQMetaObject;
class TQVariant;
class TQMetaProperty;
class TQPostEventList;
class TQSenderObjectList;
class TQObjectPrivate;
#ifndef TQT_NO_USERDATA
class TQObjectUserData;
#endif
struct TQUObject;

#ifdef USE_QT4

#include <Qt/qobject.h>
#include <Qt/qapplication.h>

#define TQT_TQOBJECT_REQUIRED_INITIALIZATION(x) connect(this, SIGNAL(destroyed(QObject*)), SLOT(tqt_handle_qt_destroyed(QObject*)));

#define TQT_TQOBJECT_CHILDEVENT_REQUIRED_HANDLER(x)										\
	if (x->type() == QEvent::ChildAdded) {											\
		QChildEvent *tqce = new QChildEvent( (QEvent::Type)TQEvent::ChildInserted, ((QChildEvent*)x)->child() );	\
		QApplication::postEvent( this, tqce );										\
	}

#define TQT_TQOBJECT_CHILDEVENT_CONDITIONAL if ((e->type() != QEvent::ChildAdded) && (e->type() != QEvent::ChildPolished))

// Defined in tqwidget.cpp
extern TQObject* convertFromTQT_BASE_OBJECT_NAMEPointer( TQT_BASE_OBJECT_NAME* x );
extern const TQObject* convertFromTQT_BASE_OBJECT_NAMEPointerConst( const TQT_BASE_OBJECT_NAME* x );

// Defined in tqobject.cpp
extern TQMetaObject* tqstaticMetaObject_helper(const QMetaObject* mobj);

class TQObject: public QObject, virtual public TQt
{
	Q_OBJECT
	TQ_OBJECT
public:
	TQObject( TQT_BASE_OBJECT_NAME *parent=0, const char *name=0 );

	TQObject *parent() const;
	TQObjectList childrenListObject();
	const TQObjectList childrenListObject() const;
	const char *tqname() const;
	const char *name() const;
	const char *name(const char *defaultName) const;
	TQObject *child( const char *objName, const char *inheritsClass = 0, bool recursiveSearch = TRUE );
	bool tqsignalsBlocked() const;

	virtual bool event( TQEvent * );
	void killTimers();

	void insertChild( TQObject *object );
	void removeChild( TQObject *object );
	bool isA(const char *classname) const;
	bool inherits( const char * ) const;

	void setName(const char *aName);

	TQObjectList *queryList( const char *inheritsClass = 0, const char *objName = 0, bool regexpMatch = TRUE, bool recursiveSearch = TRUE ) const;

	TQMetaObject *tqmetaObject() const;

	virtual TQVariant property( const char *name ) const;

	static const TQObjectList *objectTrees();
	static const TQObjectList objectTreesListObject();

	// TQt handler
	virtual bool eventFilter( TQObject *, TQEvent * );

	// Qt4 handler interface
	virtual bool eventFilter( QObject *q, QEvent *e );
	bool event( QEvent *e );

	// Interoperability
	static const TQObject& convertFromTQT_BASE_OBJECT_NAME( TQT_BASE_OBJECT_NAME& ql );

protected:
	bool checkConnectArgs(const char *signal, const TQT_BASE_OBJECT_NAME*, const char *member);

// TQt event handlers
protected:
	virtual void timerEvent( TQTimerEvent * );
	virtual void childEvent( TQChildEvent * );
	virtual void customEvent( TQCustomEvent * );

// Qt4 event handler interface
protected:
	virtual void timerEvent(QTimerEvent *e);
	virtual void childEvent(QChildEvent *e);
	virtual void customEvent(QEvent *e);

public:
	friend class TQApplication;
	friend class TQBaseApplication;
	friend class TQWidget;
	friend class TQSignal;
	friend class TQDialog;

//protected:
public:
// 	// [FIXME]
// 	// All of these need to be defined in terms of the Qt4 signal/slot mechanism somehow!
// 	// It is possible that the tqmoc needs to remain completely separate from the Qt moc
// 	// However, I am concerned that signal/slot mechanisms would then not work at all from TQt to Qt
	TQConnectionList *tqreceivers( const char* signal ) const;
	TQConnectionList *tqreceivers( int signal ) const;

	// [FIXME]
	// Verify that these are truly equivalent to their Qt3 counterparts!
	void activate_signal( int signal );
// 	void activate_signal( int signal, int );
// 	void activate_signal( int signal, double );
// 	void activate_signal( int signal, TQString );
// 	void activate_signal_bool( int signal, bool );
	void activate_signal( TQConnectionList *clist, TQUObject *o );

	static TQCString normalizeSignalSlot( const char *signalSlot );

public Q_SLOTS:
	void tqt_handle_qt_destroyed(QObject* obj);

Q_SIGNALS:
	void destroyed( TQObject* obj );

private:
	TQSignalVec *connections;
	TQSenderObjectList *senderObjects;
	mutable TQString static_object_name;

// Will these work as-is or do they need serious help?
private:
	uint	isSignal   : 1;
	uint	isWidget   : 1;
	uint	pendTimer  : 1;
	uint	blockSig   : 1;
	uint	wasDeleted : 1;
	uint	isTree : 1;
};

// Interoperability
inline static const TQObject& convertFromTQT_BASE_OBJECT_NAME( const TQT_BASE_OBJECT_NAME& qo ) {
	return (*static_cast<const TQObject*>(&qo));
}

#else // USE_QT4

#define TQT_TR_NOOP(x) (x)
#define TQT_TRANSLATE_NOOP(scope,x) (x)

class TQMetaObject;
class TQVariant;
class TQMetaProperty;
class TQPostEventList;
class TQSenderObjectList;
class TQObjectPrivate;
#ifndef TQT_NO_USERDATA
class TQObjectUserData;
#endif
struct TQUObject;

class TQ_EXPORT TQObject: public TQt
{
    TQ_OBJECT
    Q_PROPERTY( TQCString name READ name WRITE setName )

public:
    TQObject( TQObject *parent=0, const char *name=0 );
    virtual ~TQObject();

#ifdef TQ_TQDOC
    virtual const char *className() const;
    static TQString tr( const char *, const char * );
    static TQString trUtf8( const char *, const char * );
    virtual TQMetaObject *tqmetaObject() const;
#endif

    virtual bool event( TQEvent * );
    virtual bool eventFilter( TQObject *, TQEvent * );

    bool	 isA( const char * )	 const;
    bool	 inherits( const char * ) const;

    const char  *name() const;
    const char  *name( const char * defaultName ) const;

    virtual void setName( const char *name );
    bool	 isWidgetType()	  const { return isWidget; }
    bool	 highPriority()	  const { return FALSE; }

    bool	 tqsignalsBlocked()  const { return blockSig; }
    void	 blockSignals( bool b );

    int		 startTimer( int interval );
    void	 killTimer( int id );
    void	 killTimers();

    TQObject           *child( const char *objName, const char *inheritsClass = 0, bool recursiveSearch = TRUE ); //### const in 4.0
    const TQObjectList *childrenListObject() const { return childObjects; }

    static const TQObjectList *objectTrees();

    TQObjectList	      *queryList( const char *inheritsClass = 0,
				  const char *objName = 0,
				  bool regexpMatch = TRUE,
				  bool recursiveSearch = TRUE ) const;

    virtual void insertChild( TQObject * );
    virtual void removeChild( TQObject * );

    void	 installEventFilter( const TQObject * );
    void	 removeEventFilter( const TQObject * );

    static bool  connect( const TQObject *sender, const char *signal,
			  const TQObject *receiver, const char *member );
    bool	 connect( const TQObject *sender, const char *signal,
			  const char *member ) const;
    static bool  disconnect( const TQObject *sender, const char *signal,
			     const TQObject *receiver, const char *member );
    bool	 disconnect( const char *signal=0,
			     const TQObject *receiver=0, const char *member=0 );
    bool	 disconnect( const TQObject *receiver, const char *member=0 );
    static void 	 connectInternal( const TQObject *sender, int signal_index,
			  const TQObject *receiver, int membcode, int member_index );
    static bool 	 disconnectInternal( const TQObject *sender, int signal_index,
			     const TQObject *receiver, int membcode, int member_index );

    void	 dumpObjectTree();
    void	 dumpObjectInfo();

#ifndef TQT_NO_PROPERTIES
    virtual bool setProperty( const char *name, const TQVariant& value );
    virtual TQVariant property( const char *name ) const;
#endif // TQT_NO_PROPERTIES
#ifdef TQT_NO_TRANSLATION
    static TQString tr( const char *sourceText, const char * = 0);
#ifndef TQT_NO_TEXTCODEC
    static TQString trUtf8( const char *sourceText, const char * = 0);
#endif
#endif //TQT_NO_TRANSLATION

#ifndef TQT_NO_USERDATA
    static uint registerUserData();
    void setUserData( uint id, TQObjectUserData* data);
    TQObjectUserData* userData( uint id ) const;
#endif // TQT_NO_USERDATA

Q_SIGNALS:
    void	 destroyed();
    void	 destroyed( TQObject* obj );

public:
    TQObject	*parent() const { return parentObj; }

public Q_SLOTS:
    void	deleteLater();

private Q_SLOTS:
    void	 cleanupEventFilter( TQObject* );

protected:
    bool	activate_filters( TQEvent * );
    TQConnectionList *tqreceivers( const char* signal ) const;
    TQConnectionList *tqreceivers( int signal ) const;
    void	activate_signal( int signal );
    void	activate_signal( int signal, int );
    void	activate_signal( int signal, double );
    void	activate_signal( int signal, TQString );
    void	activate_signal_bool( int signal, bool );
    void 	activate_signal( TQConnectionList *clist, TQUObject *o );

    const TQObject *sender();

    virtual void timerEvent( TQTimerEvent * );
    virtual void childEvent( TQChildEvent * );
    virtual void customEvent( TQCustomEvent * );

    virtual void connectNotify( const char *signal );
    virtual void disconnectNotify( const char *signal );
    virtual bool checkConnectArgs( const char *signal, const TQObject *receiver,
				   const char *member );
    static TQCString normalizeSignalSlot( const char *signalSlot );

private:
    uint	isSignal   : 1;
    uint	isWidget   : 1;
    uint	pendTimer  : 1;
    uint	blockSig   : 1;
    uint	wasDeleted : 1;
    uint	isTree : 1;

    const char	*objname;
    TQObject	*parentObj;
    TQObjectList *childObjects;
    TQSignalVec *connections;
    TQSenderObjectList *senderObjects;
    TQObjectList *eventFilters;
    TQPostEventList *postedEvents;
    TQObjectPrivate* d;

    static TQMetaObject* staticTQtMetaObject();

    friend class TQApplication;
    friend class TQBaseApplication;
    friend class TQWidget;
    friend class TQSignal;

private:	// Disabled copy constructor and operator=
#if defined(TQ_DISABLE_COPY)
    TQObject( const TQObject & );
    TQObject &operator=( const TQObject & );
#endif
};


#ifndef TQT_NO_USERDATA
class TQ_EXPORT TQObjectUserData {
public:
    virtual ~TQObjectUserData();
};
#endif


inline bool TQObject::connect( const TQObject *sender, const char *signal,
			      const char *member ) const
{
    return connect( sender, signal, this, member );
}


inline bool TQObject::disconnect( const char *signal,
				 const TQObject *receiver, const char *member )
{
    return disconnect( this, signal, receiver, member );
}


inline bool TQObject::disconnect( const TQObject *receiver, const char *member )
{
    return disconnect( this, 0, receiver, member );
}


#ifdef TQT_NO_TRANSLATION
inline TQString TQObject::tr( const char *sourceText, const char * ) {
    return TQString::tqfromLatin1( sourceText );
}
#ifndef TQT_NO_TEXTCODEC
inline TQString TQObject::trUtf8( const char *sourceText, const char * ) {
    return TQString::fromUtf8( sourceText );
}
#endif
#endif //TQT_NO_TRANSLATION


#define TQ_DEFINED_TQOBJECT
#include "tqwinexport.h"
#endif // USE_QT4
#endif // TQOBJECT_H
