/****************************************************************************
**
** Definition of TQPen class
**
** Created : 940112
**
** Copyright (C) 2010 Timothy Pearson and (C) 1992-2008 Trolltech ASA.
**
** This file is part of the kernel module of the TQt GUI Toolkit.
**
** This file may be used under the terms of the GNU General
** Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the files LICENSE.GPL2
** and LICENSE.GPL3 included in the packaging of this file.
** Alternatively you may (at your option) use any later version
** of the GNU General Public License if such license has been
** publicly approved by Trolltech ASA (or its successors, if any)
** and the KDE Free TQt Foundation.
**
** Please review the following information to ensure GNU General
** Public Licensing requirements will be met:
** http://trolltech.com/products/qt/licenses/licensing/opensource/.
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
** or contact the sales department at sales@trolltech.com.
**
** This file may be used under the terms of the Q Public License as
** defined by Trolltech ASA and appearing in the file LICENSE.TQPL
** included in the packaging of this file.  Licensees holding valid TQt
** Commercial licenses may use this file in accordance with the TQt
** Commercial License Agreement provided with the Software.
**
** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
** herein.
**
**********************************************************************/

#ifndef TQPEN_H
#define TQPEN_H

#include "tqtglobaldefines.h"

#ifndef TQT_H
#include "tqcolor.h"
#include "tqshared.h"
#endif // TQT_H

#ifdef USE_QT4

#include <Qt/qpen.h>

#endif // USE_QT4

#ifdef USE_QT4

class TQ_EXPORT TQPen: public QPen, virtual public TQt
{
public:
	TQPen() : QPen() {}
	TQPen( Qt::PenStyle ps ) : QPen( ps ) {}
	TQPen( const QColor &color, uint width=0, Qt::PenStyle style=Qt::SolidLine ) : QPen( color, width, style ) {}
	TQPen( const QColor &color, uint width, TQt::PenStyle style ) : QPen( color, width, (Qt::PenStyle)style ) {}
	TQPen( const QColor &cl, uint w, Qt::PenStyle s, Qt::PenCapStyle c, Qt::PenJoinStyle j) : QPen( cl, w, s, c, j ) {}
	TQPen( const QPen &p ) : QPen( p ) {}

	inline void tqsetStyle( TQt::PenStyle tqps ) { setStyle((Qt::PenStyle)tqps); }
	inline void tqsetCapStyle( TQt::PenCapStyle tqps ) { setCapStyle((Qt::PenCapStyle)tqps); }
	inline void tqsetJoinStyle( TQt::PenJoinStyle tqps ) { setJoinStyle((Qt::PenJoinStyle)tqps); }

	using QPen::operator=;
};

#else // USE_QT4

class TQ_EXPORT TQPen: public TQt
{
public:
    TQPen();
    TQPen( PenStyle );
    TQPen( const TQColor &color, uint width=0, PenStyle style=SolidLine );
    TQPen( const TQColor &cl, uint w, PenStyle s, PenCapStyle c, PenJoinStyle j);
    TQPen( const TQPen & );
   ~TQPen();
    TQPen &operator=( const TQPen & );

    PenStyle	style() const		{ return data->style; }
    void	setStyle( PenStyle );
    uint	width() const		{ return data->width; }
    void	setWidth( uint );
    const TQColor &color() const		{ return data->color; }
    void	setColor( const TQColor & );
    PenCapStyle	capStyle() const;
    void	setCapStyle( PenCapStyle );
    PenJoinStyle joinStyle() const;
    void	setJoinStyle( PenJoinStyle );

    bool	operator==( const TQPen &p ) const;
    bool	operator!=( const TQPen &p ) const
					{ return !(operator==(p)); }

private:
    friend class TQPainter;
#ifdef TQ_WS_WIN
    friend class TQFontEngineWin;
#endif

    TQPen	copy()	const;
    void	detach();
    void	init( const TQColor &, uint, uint );
    struct TQPenData : public TQShared {		// pen data
	PenStyle  style;
	uint	  width;
	TQColor	  color;
	TQ_UINT16  linest;
    } *data;
};


/*****************************************************************************
  TQPen stream functions
 *****************************************************************************/
#ifndef TQT_NO_DATASTREAM
TQ_EXPORT TQDataStream &operator<<( TQDataStream &, const TQPen & );
TQ_EXPORT TQDataStream &operator>>( TQDataStream &, TQPen & );
#endif

#endif // USE_QT4

#endif // TQPEN_H
