/****************************************************************************
**
** Definition of TQPointArray class
**
** Created : 940213
**
** Copyright (C) 2010 Timothy Pearson and (C) 1992-2008 Trolltech ASA.
**
** This file is part of the kernel module of the TQt GUI Toolkit.
**
** This file may be used under the terms of the GNU General
** Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the files LICENSE.GPL2
** and LICENSE.GPL3 included in the packaging of this file.
** Alternatively you may (at your option) use any later version
** of the GNU General Public License if such license has been
** publicly approved by Trolltech ASA (or its successors, if any)
** and the KDE Free TQt Foundation.
**
** Please review the following information to ensure GNU General
** Public Licensing requirements will be met:
** http://trolltech.com/products/qt/licenses/licensing/opensource/.
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
** or contact the sales department at sales@trolltech.com.
**
** This file may be used under the terms of the Q Public License as
** defined by Trolltech ASA and appearing in the file LICENSE.TQPL
** included in the packaging of this file.  Licensees holding valid TQt
** Commercial licenses may use this file in accordance with the TQt
** Commercial License Agreement provided with the Software.
**
** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
** herein.
**
**********************************************************************/

#ifndef TQPOINTARRAY_H
#define TQPOINTARRAY_H

#include "tqtglobaldefines.h"

#ifndef TQT_H
#include "tqmemarray.h"
#include "tqpoint.h"
#endif // TQT_H

#ifdef USE_QT4

/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the Qt3Support module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QtGui/qpolygon.h>

QT_BEGIN_HEADER

QT_BEGIN_NAMESPACE

QT_MODULE(Qt3SupportLight)

class Q_COMPAT_EXPORT TQPointArray : public QPolygon, virtual public TQt
{
public:
    inline TQPointArray() : QPolygon() {}
    inline TQPointArray(const QRect &r, bool closed=false) : QPolygon(r, closed) {}
    inline TQPointArray(const QPolygon& a) : QPolygon(a) {}
    inline TQPointArray( int nPoints, const TQCOORD *points ) : QPolygon() { setPoints( nPoints, points ); }

    inline TQPointArray copy() const { return *this; }
    inline bool isNull() { return isEmpty(); }
    void makeEllipse(int x, int y, int w, int h);
#ifndef QT_NO_WMATRIX
    void makeArc(int x, int y, int w, int h, int a1, int a2);
    void makeArc(int x, int y, int w, int h, int a1, int a2, const QMatrix &matrix);
#endif
    TQPointArray cubicBezier() const;
};

QT_END_NAMESPACE

QT_END_HEADER

#else // USE_QT4

#if defined(TQ_TEMPLATEDLL)
//TQ_TEMPLATE_EXTERN template class TQ_EXPORT TQMemArray<TQPoint>;
#endif

class TQ_EXPORT TQPointArray : public TQMemArray<TQPoint>
{
public:
    TQPointArray() {}
    ~TQPointArray() {}
    TQPointArray( int size ) : TQMemArray<TQPoint>( size ) {}
    TQPointArray( const TQPointArray &a ) : TQMemArray<TQPoint>( a ) {}
    TQPointArray( const TQRect &r, bool closed=FALSE );
    TQPointArray( int nPoints, const TQCOORD *points );

    TQPointArray	 &operator=( const TQPointArray &a )
	{ return (TQPointArray&)assign( a ); }

    TQPointArray copy() const
	{ TQPointArray tmp; return *((TQPointArray*)&tmp.duplicate(*this)); }

    void    translate( int dx, int dy );
    TQRect   boundingRect() const;

    void    point( uint i, int *x, int *y ) const;
    TQPoint  point( uint i ) const;
    void    setPoint( uint i, int x, int y );
    void    setPoint( uint i, const TQPoint &p );
    bool    setPoints( int nPoints, const TQCOORD *points );
    bool    setPoints( int nPoints, int firstx, int firsty, ... );
    bool    putPoints( int index, int nPoints, const TQCOORD *points );
    bool    putPoints( int index, int nPoints, int firstx, int firsty, ... );
    bool    putPoints( int index, int nPoints,
		       const TQPointArray & from, int fromIndex=0 );

    void    makeArc( int x, int y, int w, int h, int a1, int a2 );
    void    makeEllipse( int x, int y, int w, int h );
    void    makeArc( int x, int y, int w, int h, int a1, int a2,
		     const TQWMatrix& );
#ifndef TQT_NO_BEZIER
    TQPointArray cubicBezier() const;
#endif
    void*  shortPoints( int index = 0, int nPoints = -1 ) const;
    static void cleanBuffers();

protected:
    static uint splen;
    static void* sp;
};


/*****************************************************************************
  TQPointArray stream functions
 *****************************************************************************/
#ifndef TQT_NO_DATASTREAM
TQ_EXPORT TQDataStream &operator<<( TQDataStream &, const TQPointArray & );
TQ_EXPORT TQDataStream &operator>>( TQDataStream &, TQPointArray & );
#endif

/*****************************************************************************
  Misc. TQPointArray functions
 *****************************************************************************/

inline void TQPointArray::setPoint( uint i, const TQPoint &p )
{
    setPoint( i, p.x(), p.y() );
}

#endif // USE_QT4

#endif // TQPOINTARRAY_H
