/****************************************************************************
**
** Definition of TQSessionManager class
**
** Created : 990510
**
** Copyright (C) 2010 Timothy Pearson and (C) 1992-2008 Trolltech ASA.
**
** This file is part of the kernel module of the TQt GUI Toolkit.
**
** This file may be used under the terms of the GNU General
** Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the files LICENSE.GPL2
** and LICENSE.GPL3 included in the packaging of this file.
** Alternatively you may (at your option) use any later version
** of the GNU General Public License if such license has been
** publicly approved by Trolltech ASA (or its successors, if any)
** and the KDE Free TQt Foundation.
**
** Please review the following information to ensure GNU General
** Public Licensing requirements will be met:
** http://trolltech.com/products/qt/licenses/licensing/opensource/.
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
** or contact the sales department at sales@trolltech.com.
**
** This file may be used under the terms of the Q Public License as
** defined by Trolltech ASA and appearing in the file LICENSE.TQPL
** included in the packaging of this file.  Licensees holding valid TQt
** Commercial licenses may use this file in accordance with the TQt
** Commercial License Agreement provided with the Software.
**
** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
** herein.
**
**********************************************************************/

#ifndef TQSESSIONMANAGER_H
#define TQSESSIONMANAGER_H

#include "tqtglobaldefines.h"

#ifndef TQT_H
#include "tqobject.h"
#include "tqwindowdefs.h"
#include "tqstring.h"
#include "tqstringlist.h"
#endif // TQT_H

#ifdef USE_QT4
#include "tqapplication.h"
// Nasty, nasty horrid HACK to get access to QSessionManager's private members
// This is TERRIBLE and I wish there was a way around it
// This is a good example of the new, broken & irritating Qt4 API,
// and the corresponding loss in functionality versus Qt3.
// See also tqrect.cpp
#define private protected
#include <Qt/qsessionmanager.h>
#undef private
#endif // USE_QT4

#ifndef TQT_NO_SESSIONMANAGER

#ifdef USE_QT4

class TQ_EXPORT TQSessionManager : public QSessionManager, virtual public TQt
{
	Q_OBJECT
	TQ_OBJECT
	TQSessionManager( QApplication *app, QString id, QString key ) : QSessionManager( app, id, key ) {}

	inline TQStringList restartCommand() const {
		QStringList ql = QSessionManager::restartCommand();
		TQStringList tqsl;
		tqsl.clear();
		for (int i = 0; i < ql.size(); ++i) tqsl.append(ql.at(i));
		return tqsl;
	}

	inline TQStringList discardCommand() const {
		QStringList ql = QSessionManager::discardCommand();
		TQStringList tqsl;
		tqsl.clear();
		for (int i = 0; i < ql.size(); ++i) tqsl.append(ql.at(i));
		return tqsl;
	}

	inline void setRestartCommand( const TQStringList& tqsl ) {
		QStringList ql;
		ql.clear();
		for ( TQStringList::ConstIterator it = tqsl.begin(); it != tqsl.end(); ++it ) ql.append(*it);
		QSessionManager::setRestartCommand(ql);
	}

	inline void setDiscardCommand( const TQStringList& tqsl ) {
		QStringList ql;
		ql.clear();
		for ( TQStringList::ConstIterator it = tqsl.begin(); it != tqsl.end(); ++it ) ql.append(*it);
		QSessionManager::setDiscardCommand(ql);
	}

	// Interoperability
	static TQSessionManager& convertFromQSessionManager( QSessionManager& qsm );
	static const TQSessionManager& convertFromQSessionManager( const QSessionManager& qsm );
};

// Interoperability
inline static TQSessionManager& convertFromQSessionManager( QSessionManager& qsm ) {
	return (*static_cast<TQSessionManager*>(&qsm));
}
inline static const TQSessionManager& convertFromQSessionManager( const QSessionManager& qsm ) {
	return (*static_cast<const TQSessionManager*>(&qsm));
}

#else // USE_QT4

class TQSessionManagerData;

class TQ_EXPORT  TQSessionManager : public TQObject
{
    TQ_OBJECT
    TQSessionManager( TQApplication *app, TQString &id, TQString &key );
    ~TQSessionManager();
public:
    TQString sessionId() const;
    TQString sessionKey() const;
#if defined(TQ_WS_X11) || defined(TQ_WS_MAC)
    void* handle() const;
#endif

    bool allowsInteraction();
    bool allowsErrorInteraction();
    void release();

    void cancel();

    enum RestartHint {
	RestartIfRunning,
	RestartAnyway,
	RestartImmediately,
	RestartNever
    };
    void setRestartHint( RestartHint );
    RestartHint restartHint() const;

    void setRestartCommand( const TQStringList& );
    TQStringList restartCommand() const;
    void setDiscardCommand( const TQStringList& );
    TQStringList discardCommand() const;

    void setManagerProperty( const TQString& name, const TQString& value );
    void setManagerProperty( const TQString& name, const TQStringList& value );

    bool isPhase2() const;
    void requestPhase2();

private:
    friend class TQApplication;
    friend class TQBaseApplication;
    TQSessionManagerData* d;
};

#endif // USE_QT4
#endif // TQT_NO_SESSIONMANAGER
#endif // TQSESSIONMANAGER_H
