/****************************************************************************
**
** Definition of Windows-like style class
**
** Created : 981231
**
** Copyright (C) 2010 Timothy Pearson and (C) 1992-2008 Trolltech ASA.
**
** This file is part of the widgets module of the TQt GUI Toolkit.
**
** This file may be used under the terms of the GNU General
** Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the files LICENSE.GPL2
** and LICENSE.GPL3 included in the packaging of this file.
** Alternatively you may (at your option) use any later version
** of the GNU General Public License if such license has been
** publicly approved by Trolltech ASA (or its successors, if any)
** and the KDE Free TQt Foundation.
**
** Please review the following information to ensure GNU General
** Public Licensing requirements will be met:
** http://trolltech.com/products/qt/licenses/licensing/opensource/.
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
** or contact the sales department at sales@trolltech.com.
**
** This file may be used under the terms of the Q Public License as
** defined by Trolltech ASA and appearing in the file LICENSE.TQPL
** included in the packaging of this file.  Licensees holding valid TQt
** Commercial licenses may use this file in accordance with the TQt
** Commercial License Agreement provided with the Software.
**
** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
** herein.
**
**********************************************************************/

#ifndef TQWINDOWSSTYLE_H
#define TQWINDOWSSTYLE_H

#ifndef TQT_H
#include "tqcommonstyle.h"
#endif // TQT_H

#if !defined(TQT_NO_STYLE_WINDOWS) || defined(TQT_PLUGIN)

#if defined(TQT_PLUGIN)
#define TQ_EXPORT_STYLE_WINDOWS
#else
#define TQ_EXPORT_STYLE_WINDOWS TQ_EXPORT
#endif


class TQ_EXPORT_STYLE_WINDOWS TQWindowsStyle : public TQCommonStyle
{
    Q_OBJECT
    TQ_OBJECT
public:
    TQWindowsStyle();
    ~TQWindowsStyle();

    void polish(TQApplication*);
    void unPolish(TQApplication*);

    void polish(TQWidget*);
    void unPolish(TQWidget*);

    void polish( TQPalette & );

    virtual void polishPopupMenu( TQPopupMenu* );

    // new stuff
    void tqdrawPrimitiveBase( TQ_PrimitiveElement pe,
			TQPainter *p,
			const TQRect &r,
			const TQColorGroup &cg,
			SFlags flags = Style_Default,
			const TQStyleOption& = TQStyleOption::Default ) const;

    void tqdrawControl( TQ_ControlElement element,
		      TQPainter *p,
		      const TQWidget *widget,
		      const TQRect &r,
		      const TQColorGroup &cg,
		      SFlags flags = Style_Default,
		      const TQStyleOption& = TQStyleOption::Default ) const;

    void tqdrawComplexControl( TQ_ComplexControl control,
			     TQPainter* p,
			     const TQWidget* widget,
			     const TQRect& r,
			     const TQColorGroup& cg,
			     SFlags flags = Style_Default,
#ifdef TQ_TQDOC
			     SCFlags sub = SC_All,
#else
			     SCFlags sub = (uint)SC_All,
#endif
			     SCFlags subActive = SC_None,
			     const TQStyleOption& = TQStyleOption::Default ) const;

    int tqpixelMetric( PixelMetric metric,
		     const TQWidget *widget = 0 ) const;

    TQSize tqsizeFromContents( ContentsType contents,
			    const TQWidget *widget,
			    const TQSize &contentsSize,
			    const TQStyleOption& = TQStyleOption::Default ) const;

    int tqstyleHint(TQ_StyleHint sh, const TQWidget *, const TQStyleOption & = TQStyleOption::Default,
		  TQStyleHintReturn* = 0) const;

    TQPixmap stylePixmap( StylePixmap stylepixmap,
			 const TQWidget *widget = 0,
			 const TQStyleOption& = TQStyleOption::Default ) const;

    TQRect subRect( SubRect r, const TQWidget *widget ) const;

private:
    class Private;
    Private *d;

    // Disabled copy constructor and operator=
#if defined(TQ_DISABLE_COPY)
    TQWindowsStyle( const TQWindowsStyle & );
    TQWindowsStyle& operator=( const TQWindowsStyle & );
#endif
};

#endif // TQT_NO_STYLE_WINDOWS

#endif // TQWINDOWSSTYLE_H
