/****************************************************************************
**
** Definition of TQJpUnicodeConv class
**
** Created : 990225
**
** Copyright (C) 2010 Timothy Pearson and (C) 1992-2008 Trolltech ASA.
**
** This file is part of the tools module of the TQt GUI Toolkit.
**
** This file may be used under the terms of the GNU General
** Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the files LICENSE.GPL2
** and LICENSE.GPL3 included in the packaging of this file.
** Alternatively you may (at your option) use any later version
** of the GNU General Public License if such license has been
** publicly approved by Trolltech ASA (or its successors, if any)
** and the KDE Free TQt Foundation.
**
** Please review the following information to ensure GNU General
** Public Licensing requirements will be met:
** http://trolltech.com/products/qt/licenses/licensing/opensource/.
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
** or contact the sales department at sales@trolltech.com.
**
** This file may be used under the terms of the Q Public License as
** defined by Trolltech ASA and appearing in the file LICENSE.TQPL
** included in the packaging of this file.  Licensees holding valid TQt
** Commercial licenses may use this file in accordance with the TQt
** Commercial License Agreement provided with the Software.
**
** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
** herein.
**
**********************************************************************/

// Most of the code here was originally written by Serika Kurusugawa
// a.k.a. Junji Takagi, and is included in TQt with the author's permission,
// and the grateful thanks of the Trolltech team.

/*
 * Copyright (C) 1999 Serika Kurusugawa, All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSETQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef TQJPUNICODE_H
#define TQJPUNICODE_H

#ifndef TQT_H
#include "tqglobal.h"
#endif // TQT_H

#ifndef TQT_NO_BIG_CODECS

#if defined(TQT_PLUGIN)
#define TQ_EXPORT_CODECS_JP
#else
#define TQ_EXPORT_CODECS_JP TQ_EXPORT
#endif

class TQ_EXPORT_CODECS_JP TQJpUnicodeConv {
public:
    enum Rules {
	// "ASCII" is ANSI X.3.4-1986, a.k.a. US-ASCII here.
	Default			= 0x0000,

	Unicode			= 0x0001,
	Unicode_JISX0201		= 0x0001,
	Unicode_ASCII 		= 0x0002,
	JISX0221_JISX0201 	= 0x0003,
	JISX0221_ASCII		= 0x0004,
	Sun_JDK117             	= 0x0005,
	Microsoft_CP932        	= 0x0006,

	NEC_VDC	       	= 0x0100,		// NEC Vender Defined Char
	UDC	       		= 0x0200,		// User Defined Char
	IBM_VDC		= 0x0400		// IBM Vender Defined Char
    };
    static TQJpUnicodeConv *newConverter(int rule);

    virtual uint asciiToUnicode(uint h, uint l) const;
    /*virtual*/ uint jisx0201ToUnicode(uint h, uint l) const;
    virtual uint jisx0201LatinToUnicode(uint h, uint l) const;
    /*virtual*/ uint jisx0201KanaToUnicode(uint h, uint l) const;
    virtual uint jisx0208ToUnicode(uint h, uint l) const;
    virtual uint jisx0212ToUnicode(uint h, uint l) const;

    uint asciiToUnicode(uint ascii) const {
	return asciiToUnicode((ascii & 0xff00) >> 8, (ascii & 0x00ff));
    }
    uint jisx0201ToUnicode(uint jis) const {
	return jisx0201ToUnicode((jis & 0xff00) >> 8, (jis & 0x00ff));
    }
    uint jisx0201LatinToUnicode(uint jis) const {
	return jisx0201LatinToUnicode((jis & 0xff00) >> 8, (jis & 0x00ff));
    }
    uint jisx0201KanaToUnicode(uint jis) const {
	return jisx0201KanaToUnicode((jis & 0xff00) >> 8, (jis & 0x00ff));
    }
    uint jisx0208ToUnicode(uint jis) const {
	return jisx0208ToUnicode((jis & 0xff00) >> 8, (jis & 0x00ff));
    }
    uint jisx0212ToUnicode(uint jis) const {
	return jisx0212ToUnicode((jis & 0xff00) >> 8, (jis & 0x00ff));
    }

    virtual uint tqunicodeToAscii(uint h, uint l) const;
    /*virtual*/ uint tqunicodeToJisx0201(uint h, uint l) const;
    virtual uint tqunicodeToJisx0201Latin(uint h, uint l) const;
    /*virtual*/ uint tqunicodeToJisx0201Kana(uint h, uint l) const;
    virtual uint tqunicodeToJisx0208(uint h, uint l) const;
    virtual uint tqunicodeToJisx0212(uint h, uint l) const;

    uint tqunicodeToAscii(uint tqunicode) const {
	return tqunicodeToAscii((tqunicode & 0xff00) >> 8, (tqunicode & 0x00ff));
    }
    uint tqunicodeToJisx0201(uint tqunicode) const {
	return tqunicodeToJisx0201((tqunicode & 0xff00) >> 8, (tqunicode & 0x00ff));
    }
    uint tqunicodeToJisx0201Latin(uint tqunicode) const {
	return tqunicodeToJisx0201Latin((tqunicode & 0xff00) >> 8, (tqunicode & 0x00ff));
    }
    uint tqunicodeToJisx0201Kana(uint tqunicode) const {
	return tqunicodeToJisx0201Kana((tqunicode & 0xff00) >> 8, (tqunicode & 0x00ff));
    }
    uint tqunicodeToJisx0208(uint tqunicode) const {
	return tqunicodeToJisx0208((tqunicode & 0xff00) >> 8, (tqunicode & 0x00ff));
    }
    uint tqunicodeToJisx0212(uint tqunicode) const {
	return tqunicodeToJisx0212((tqunicode & 0xff00) >> 8, (tqunicode & 0x00ff));
    }

    uint sjisToUnicode(uint h, uint l) const;
    uint tqunicodeToSjis(uint h, uint l) const;
    uint sjisibmvdcToUnicode(uint h, uint l) const;
    uint tqunicodeToSjisibmvdc(uint h, uint l) const;
    uint cp932ToUnicode(uint h, uint l) const;
    uint tqunicodeToCp932(uint h, uint l) const;

    uint sjisToUnicode(uint sjis) const {
	return sjisToUnicode((sjis & 0xff00) >> 8, (sjis & 0x00ff));
    }
    uint tqunicodeToSjis(uint tqunicode) const {
	return tqunicodeToSjis((tqunicode & 0xff00) >> 8, (tqunicode & 0x00ff));
    }

protected:
    TQJpUnicodeConv(int r) : rule(r) {}

private:
    int rule;
};

#endif // TQT_NO_BIG_CODECS
#endif /* TQJPUNICODE_H */
