#include "tqtglobaldefines.h"

#ifdef USE_QT4

/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the Qt3Support module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef TQSIMPLERICHTEXT_H
#define TQSIMPLERICHTEXT_H

#include <QtCore/qnamespace.h>
#include <QtCore/qstring.h>
#include <QtGui/qregion.h>
#include <QtGui/qcolor.h>

#include "tqpalette.h"
#include "tqpainter.h"

QT_BEGIN_HEADER

QT_BEGIN_NAMESPACE

QT_MODULE(Qt3SupportLight)

#ifndef QT_NO_RICHTEXT

class QPainter;
class QWidget;
class TQStyleSheet;
class QBrush;
class TQMimeSourceFactory;
class TQSimpleRichTextData;

class Q_COMPAT_EXPORT TQSimpleRichText
{
public:
    TQSimpleRichText(const QString& text, const QFont& fnt,
                     const QString& context = QString(), const TQStyleSheet* sheet = 0);
    TQSimpleRichText(const QString& text, const QFont& fnt,
                     const QString& context, const TQStyleSheet *sheet,
                     const TQMimeSourceFactory* factory, int pageBreak = -1,
                     const QColor& linkColor = Qt::blue, bool linkUnderline = true);
    ~TQSimpleRichText();

    void setWidth(int);
    void setWidth(QPainter*, int);
    void setDefaultFont(const QFont &f);
    int width() const;
    int widthUsed() const;
    int height() const;
    void adjustSize();

    void draw(QPainter* p, int x, int y, const QRect& clipRect,
               const TQColorGroup& cg, const QBrush* paper = 0) const;

    void draw(QPainter* p, int x, int y, const QRegion& clipRegion,
               const TQColorGroup& cg, const QBrush* paper = 0) const {
        draw(p, x, y, clipRegion.boundingRect(), cg, paper);
    }

    TQString context() const;
    TQString anchorAt(const QPoint& pos) const;

    bool inText(const QPoint& pos) const;

private:
    Q_DISABLE_COPY(TQSimpleRichText)

    TQSimpleRichTextData* d;
};

#endif // QT_NO_RICHTEXT

QT_END_NAMESPACE

QT_END_HEADER

#endif // TQSIMPLERICHTEXT_H

#else // USE_QT4

/****************************************************************************
**
** Definition of the TQSimpleRichText class
**
** Created : 990101
**
** Copyright (C) 2010 Timothy Pearson and (C) 1992-2008 Trolltech ASA.
**
** This file is part of the kernel module of the TQt GUI Toolkit.
**
** This file may be used under the terms of the GNU General
** Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the files LICENSE.GPL2
** and LICENSE.GPL3 included in the packaging of this file.
** Alternatively you may (at your option) use any later version
** of the GNU General Public License if such license has been
** publicly approved by Trolltech ASA (or its successors, if any)
** and the KDE Free TQt Foundation.
**
** Please review the following information to ensure GNU General
** Public Licensing requirements will be met:
** http://trolltech.com/products/qt/licenses/licensing/opensource/.
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
** or contact the sales department at sales@trolltech.com.
**
** This file may be used under the terms of the Q Public License as
** defined by Trolltech ASA and appearing in the file LICENSE.TQPL
** included in the packaging of this file.  Licensees holding valid TQt
** Commercial licenses may use this file in accordance with the TQt
** Commercial License Agreement provided with the Software.
**
** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
** herein.
**
**********************************************************************/

#ifndef TQSIMPLERICHTEXT_H
#define TQSIMPLERICHTEXT_H

#ifndef TQT_H
#include "tqnamespace.h"
#include "tqstring.h"
#include "tqregion.h"
#endif // TQT_H

#ifndef TQT_NO_RICHTEXT

class TQ_EXPORT TQSimpleRichText : public QSimpleRichText
{
public:
    TQSimpleRichText( const QString& text, const QFont& fnt, const QString& context = TQString::null, const QStyleSheet* sheet = 0) : QSimpleRichText( text, fnt, context, sheet ) {}
    TQSimpleRichText( const TQString& text, const TQFont& fnt, const TQString& context,  const TQStyleSheet* sheet, const TQMimeSourceFactory* factory, int pageBreak = -1, const TQColor& linkColor = TQt::blue, bool linkUnderline = TRUE ) : QSimpleRichText( text, fnt, context, sheet, factory, pageBreak, linkColor, linkUnderline ) {}
};

#else // USE_QT4

class TQPainter;
class TQWidget;
class TQStyleSheet;
class TQBrush;
class TQMimeSourceFactory;
class TQSimpleRichTextData;

class TQ_EXPORT TQSimpleRichText
{
public:
    TQSimpleRichText( const TQString& text, const TQFont& fnt,
		     const TQString& context = TQString::null, const TQStyleSheet* sheet = 0);
    TQSimpleRichText( const TQString& text, const TQFont& fnt,
		     const TQString& context,  const TQStyleSheet* sheet,
		     const TQMimeSourceFactory* factory, int pageBreak = -1,
		     const TQColor& linkColor = TQt::blue, bool linkUnderline = TRUE );
    ~TQSimpleRichText();

    void setWidth( int );
    void setWidth( TQPainter*, int );
    void setDefaultFont( const TQFont &f );
    int width() const;
    int widthUsed() const;
    int height() const;
    void adjustSize();

    void draw( TQPainter* p,  int x, int y, const QRect& clipRect,
	       const TQColorGroup& cg, const TQBrush* paper = 0) const;

    // obsolete
    void draw( TQPainter* p,  int x, int y, const TQRegion& clipRegion,
	       const TQColorGroup& cg, const TQBrush* paper = 0) const {
	draw( p, x, y, clipRegion.boundingRect(), cg, paper );
    }

    TQString context() const;
    TQString anchorAt( const TQPoint& pos ) const;

    bool inText( const TQPoint& pos ) const;

private:
    TQSimpleRichTextData* d;

private:	// Disabled copy constructor and operator=
#if defined(TQ_DISABLE_COPY)
    TQSimpleRichText( const TQSimpleRichText & );
    TQSimpleRichText &operator=( const TQSimpleRichText & );
#endif
};

#endif // TQT_NO_RICHTEXT

#endif // TQSIMPLERICHTEXT_H

#endif // USE_QT4