/****************************************************************************
**
** Definition of TQFile class
**
** Created : 930831
**
** Copyright (C) 2010 Timothy Pearson and (C) 1992-2008 Trolltech ASA.
**
** This file is part of the tools module of the TQt GUI Toolkit.
**
** This file may be used under the terms of the GNU General
** Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the files LICENSE.GPL2
** and LICENSE.GPL3 included in the packaging of this file.
** Alternatively you may (at your option) use any later version
** of the GNU General Public License if such license has been
** publicly approved by Trolltech ASA (or its successors, if any)
** and the KDE Free TQt Foundation.
**
** Please review the following information to ensure GNU General
** Public Licensing requirements will be met:
** http://trolltech.com/products/qt/licenses/licensing/opensource/.
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
** or contact the sales department at sales@trolltech.com.
**
** This file may be used under the terms of the Q Public License as
** defined by Trolltech ASA and appearing in the file LICENSE.TQPL
** included in the packaging of this file.  Licensees holding valid TQt
** Commercial licenses may use this file in accordance with the TQt
** Commercial License Agreement provided with the Software.
**
** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
** herein.
**
**********************************************************************/

#ifndef TQFILE_H
#define TQFILE_H

#include "tqtglobaldefines.h"

#ifndef TQT_H
#include "tqiodevice.h"
#include "tqstring.h"
#include <stdio.h>
#endif // TQT_H

#ifdef USE_QT4

#include <Qt/qfile.h>

#endif // USE_QT4

class TQDir;
class TQFilePrivate;

#ifdef USE_QT4

class TQ_EXPORT TQFile : public QFile
{
public:
	TQFile() : QFile() {}
	TQFile( const QString &name ) : QFile( name ) {}

	bool open( int aFlags, FILE * f ) { return QFile::open( f, (QIODevice::OpenModeFlag)aFlags ); }
	bool open( int aFlags, int f ) { return QFile::open( f, (QIODevice::OpenModeFlag)aFlags ); }
	TQString name() const { return fileName(); }
	void setName( const QString &name ) { setFileName(name); }
	bool open( int om ) { return QFile::open((OpenMode)om); }
	int state() const { return isOpen() ? 0x1000 : 0; }

	inline TQ_LONG readLine( char *data, TQ_ULONG maxlen ) { return QFile::readLine(data, maxlen); }
	TQ_LONG readLine( TQString &, TQ_ULONG maxlen );

	TQIODevice::Status status() const {
#if !defined(QT_NO_QOBJECT)
		const QFile *f = qobject_cast<const QFile *>(this);
		if (f) return (int) f->error();
#endif
		return isOpen() ? 0 /* IO_Ok */ : 8 /* IO_UnspecifiedError */;
	}

	inline TQIODevice::Offset at() const { return TQT_TQIODEVICE_CONST(this)->at(); }
	inline bool at(TQIODevice::Offset offset) { return TQT_TQIODEVICE(this)->at(offset); }
	inline TQIODevice::Offset tqat() const { return TQT_TQIODEVICE_CONST(this)->at(); }
	inline bool tqat(TQIODevice::Offset offset) { return TQT_TQIODEVICE(this)->at(offset); }

	inline qint64 readBlock(char *data, quint64 maxlen) { return read(data, maxlen); }
	inline qint64 writeBlock(const char *data, quint64 len) { return write(data, len); }
	inline qint64 writeBlock(const QByteArray &data) { return write(data); }

	inline int getch() { char c; return getChar(&c) ? int(uchar(c)) : -1; }
	inline int putch(int c) { return putChar(char(c)) ? int(uchar(c)) : -1; }
	inline int ungetch(int c) { ungetChar(uchar(c)); return c; }

	inline bool isDirectAccess() const { return !isSequential(); }
	inline bool isSequentialAccess() const { return isSequential(); }
	inline bool isCombinedAccess() const { return false; }
	inline bool isBuffered() const { return true; }
	inline bool isRaw() const { return false; }
	inline bool isSynchronous() const { return true; }
	inline bool isAsynchronous() const { return false; }
	inline bool isTranslated() const { return (openMode() & Text) != 0; }
	inline bool isInactive() const { return !isOpen(); }

// 	inline bool open( int mode ) = 0;

protected:
        void setqStatus( int ) { std::cout << "[WARNING] TQIODevice::setqStatus() UNIMPLEMENTED\n\r"; }
        void resetqStatus() { TQT_TQIODEVICE(this)->resetStatus(); }

	void setFlags( int f ) { setOpenMode((QIODevice::OpenModeFlag)f); }
	void setType( int ) { printf("[WARNING] TQIODevice::setType() unimplemented\n\r"); }
	void setMode( int f ) { setOpenMode((QIODevice::OpenModeFlag)f); }
	void setState( int state ) {
		switch (state) {
			case (IO_Open):
				open(openMode());
				break;
		}
	}
};

#else // USE_QT4

class TQ_EXPORT TQFile : public TQIODevice			// file I/O tqdevice class
{
public:
    TQFile();
    TQFile( const TQString &name );
   ~TQFile();

    TQString	name()	const;
    void	setName( const TQString &name );

    typedef TQCString (*EncoderFn)( const TQString &fileName );
    typedef TQString (*DecoderFn)( const TQCString &localfileName );
    static TQCString encodeName( const TQString &fileName );
    static TQString decodeName( const TQCString &localFileName );
    static void setEncodingFunction( EncoderFn );
    static void setDecodingFunction( DecoderFn );

    bool	exists()   const;
    static bool exists( const TQString &fileName );

    bool	remove();
    static bool remove( const TQString &fileName );

    bool	open( int );
    bool	open( int, FILE * );
    bool	open( int, int );
    void	close();
    void	flush();

    Offset	size() const;
    Offset	at() const;
    bool	at( Offset );
    bool	atEnd() const;

    TQ_LONG	readBlock( char *data, TQ_ULONG len );
    TQ_LONG	writeBlock( const char *data, TQ_ULONG len );
    TQ_LONG	writeBlock( const TQByteArray& data )
		      { return TQIODevice::writeBlock(data); }
    TQ_LONG	readLine( char *data, TQ_ULONG maxlen );
    TQ_LONG	readLine( TQString &, TQ_ULONG maxlen );

    int		getch();
    int		putch( int );
    int		ungetch( int );

    int		handle() const;

    TQString	errorString() const; // ### TQt 4: move into TQIODevice

protected:
    void	setErrorString( const TQString& ); // ### TQt 4: move into TQIODevice
    TQString	fn;
    FILE       *fh;
    int		fd;
    Offset	length;
    bool	ext_f;
    TQFilePrivate *d; // ### TQt 4: make private

private:
    void	init();
    void	setErrorStringErrno( int );
    TQCString	ungetchBuffer;

private:	// Disabled copy constructor and operator=
#if defined(TQ_DISABLE_COPY)
    TQFile( const TQFile & );
    TQFile &operator=( const TQFile & );
#endif
};


inline TQString TQFile::name() const
{ return fn; }

inline TQIODevice::Offset TQFile::at() const
{ return ioIndex; }

#endif // USE_QT4

#endif // TQFILE_H
