/****************************************************************************
**
** Definition of TQIODevice class
**
** Created : 940913
**
** Copyright (C) 2010 Timothy Pearson and (C) 1992-2008 Trolltech ASA.
**
** This file is part of the tools module of the TQt GUI Toolkit.
**
** This file may be used under the terms of the GNU General
** Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the files LICENSE.GPL2
** and LICENSE.GPL3 included in the packaging of this file.
** Alternatively you may (at your option) use any later version
** of the GNU General Public License if such license has been
** publicly approved by Trolltech ASA (or its successors, if any)
** and the KDE Free TQt Foundation.
**
** Please review the following information to ensure GNU General
** Public Licensing requirements will be met:
** http://trolltech.com/products/qt/licenses/licensing/opensource/.
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
** or contact the sales department at sales@trolltech.com.
**
** This file may be used under the terms of the Q Public License as
** defined by Trolltech ASA and appearing in the file LICENSE.TQPL
** included in the packaging of this file.  Licensees holding valid TQt
** Commercial licenses may use this file in accordance with the TQt
** Commercial License Agreement provided with the Software.
**
** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
** herein.
**
**********************************************************************/

#ifndef TQIODEVICE_H
#define TQIODEVICE_H

#include "tqtglobaldefines.h"

#ifndef TQT_H
#include "tqglobal.h"
#include "tqcstring.h"
#endif // TQT_H

#ifdef USE_QT4

#include <Qt/qiodevice.h>
#include <Qt/qfile.h>
#include <iostream>

#endif // USE_QT4

#ifdef USE_QT4

#define IO_Direct		0x0100		// direct access tqdevice
#define IO_Sequential		0x0200		// sequential access tqdevice
#define IO_Combined		0x0300		// combined direct/sequential
#define IO_TypeMask		0x0f00

// IO handling modes

#define IO_Raw			0x0040		// raw access (not buffered)
#define IO_Async		0x0080		// asynchronous mode

// IO tqdevice open modes

#define IO_ReadOnly		(QIODevice::OpenModeFlag)0x0001		// readable tqdevice
#define IO_WriteOnly		(QIODevice::OpenModeFlag)0x0002		// writable tqdevice
#define IO_ReadWrite		(QIODevice::OpenModeFlag)0x0003		// read+write tqdevice
#define IO_Append		(QIODevice::OpenModeFlag)0x0004		// append
#define IO_Truncate		(QIODevice::OpenModeFlag)0x0008		// truncate tqdevice
#define IO_Translate		(QIODevice::OpenModeFlag)0x0010		// translate CR+LF
#define IO_ModeMask		(QIODevice::OpenModeFlag)0x00ff

// IO tqdevice state

#define IO_Open			0x1000		// tqdevice is open
#define IO_StateMask		0xf000

// IO tqdevice status

#define IO_Ok			0
#define IO_ReadError		1		// read error
#define IO_WriteError		2		// write error
#define IO_FatalError		3		// fatal unrecoverable error
#define IO_ResourceError	4		// resource limitation
#define IO_OpenError		5		// cannot open tqdevice
#define IO_ConnectError		5		// cannot connect to tqdevice
#define IO_AbortError		6		// abort error
#define IO_TimeOutError		7		// time out
#define IO_UnspecifiedError	8		// unspecified error

class TQ_EXPORT TQIODevice : public QIODevice
{
public:
#if defined(TQT_ABI_QT4)
    typedef TQ_LLONG Offset;
#else
    typedef TQ_ULONG Offset;
#endif

	TQIODevice() : QIODevice() {}

	inline int state() const { return isOpen() ? 0x1000 : 0; }
	inline int mode() const { return (int) openMode(); }
	inline int flags() const { return (int) openMode(); }
	inline bool tqopen( int mode ) { return open((OpenModeFlag)mode); }

	inline Offset at() const { return pos(); }
	inline bool at(Offset offset) { return seek(offset); }
	inline Offset tqat() const { return pos(); }
	inline bool tqat(Offset offset) { return seek(offset); }

// 	virtual inline qint64 readBlock(char *data, quint64 maxlen) { return read(data, maxlen); }
// 	virtual inline qint64 writeBlock(const char *data, quint64 len) { return write(data, len); }
// 	virtual inline qint64 writeBlock(const QByteArray &data) { return write(data); }
	inline qint64 readBlock(char *data, quint64 maxlen) { return read(data, maxlen); }
	inline qint64 writeBlock(const char *data, quint64 len) { return write(data, len); }
	inline qint64 writeBlock(const QByteArray &data) { return write(data); }

// 	virtual inline qint64 readData ( char * data, qint64 maxSize ) { return readBlock(data, maxSize); }
// 	virtual inline qint64 writeData ( const char * data, qint64 maxSize ) { return writeBlock(data, maxSize); }

	inline int getch() { char c; return getChar(&c) ? int(uchar(c)) : -1; }
	inline int putch(int c) { return putChar(char(c)) ? int(uchar(c)) : -1; }
	inline int ungetch(int c) { ungetChar(uchar(c)); return c; }
	virtual void flush() = 0;

	inline bool isDirectAccess() const { return !isSequential(); }
	inline bool isSequentialAccess() const { return isSequential(); }
	inline bool isCombinedAccess() const { return false; }
	inline bool isBuffered() const { return true; }
	inline bool isRaw() const { return false; }
	inline bool isSynchronous() const { return true; }
	inline bool isAsynchronous() const { return false; }
	inline bool isTranslated() const { return (openMode() & Text) != 0; }
	inline bool isInactive() const { return !isOpen(); }

// 	inline bool open( int mode ) = 0;

protected:
	void setFlags( int f ) { setOpenMode((QIODevice::OpenModeFlag)f); }
	void setType( int ) { printf("[WARNING] TQIODevice::setType() unimplemented\n\r"); }
	void setMode( int f ) { setOpenMode((QIODevice::OpenModeFlag)f); }
	void setState( int state ) {
		switch (state) {
			case (IO_Open):
				QIODevice::open(openMode());
				break;
		}
	}

public:
	typedef int Status;
	Status status() const {
#if !defined(QT_NO_QOBJECT)
		const QFile *f = qobject_cast<const QFile *>(this);
		if (f) return (int) f->error();
#endif
		return isOpen() ? 0 /* IO_Ok */ : 8 /* IO_UnspecifiedError */;
	}
	void resetStatus() {
#if !defined(QT_NO_QOBJECT)
		QFile *f = qobject_cast<QFile *>(this);
		if (f) f->unsetError();
#endif
	}

	void setqStatus( int ) { std::cout << "[WARNING] TQIODevice::setqStatus() UNIMPLEMENTED\n\r"; }
	void resetqStatus() { resetStatus(); }

protected:
	friend class TQFile;
	friend class TQIODeviceSource;
	friend class TQImageIO;
	friend class TQBuffer;

public:
	// Interoperability
	static TQIODevice& convertFromQIODevice( QIODevice& qiod );
};

// Interoperability
inline static TQIODevice& convertFromQIODevice( QIODevice& qiod ) {
	return (*static_cast<TQIODevice*>(&qiod));
}

#else // USE_QT4

// IO tqdevice access types

#define IO_Direct		0x0100		// direct access tqdevice
#define IO_Sequential		0x0200		// sequential access tqdevice
#define IO_Combined		0x0300		// combined direct/sequential
#define IO_TypeMask		0x0f00

// IO handling modes

#define IO_Raw			0x0040		// raw access (not buffered)
#define IO_Async		0x0080		// asynchronous mode

// IO tqdevice open modes

#define IO_ReadOnly		0x0001		// readable tqdevice
#define IO_WriteOnly		0x0002		// writable tqdevice
#define IO_ReadWrite		0x0003		// read+write tqdevice
#define IO_Append		0x0004		// append
#define IO_Truncate		0x0008		// truncate tqdevice
#define IO_Translate		0x0010		// translate CR+LF
#define IO_ModeMask		0x00ff

// IO tqdevice state

#define IO_Open			0x1000		// tqdevice is open
#define IO_StateMask		0xf000

// IO tqdevice status

#define IO_Ok			0
#define IO_ReadError		1		// read error
#define IO_WriteError		2		// write error
#define IO_FatalError		3		// fatal unrecoverable error
#define IO_ResourceError	4		// resource limitation
#define IO_OpenError		5		// cannot open tqdevice
#define IO_ConnectError		5		// cannot connect to tqdevice
#define IO_AbortError		6		// abort error
#define IO_TimeOutError		7		// time out
#define IO_UnspecifiedError	8		// unspecified error


class TQ_EXPORT TQIODevice
{
public:
#if defined(TQT_ABI_QT4)
    typedef TQ_LLONG Offset;
#else
    typedef TQ_ULONG Offset;
#endif

    TQIODevice();
    virtual ~TQIODevice();

    int		 flags()  const { return ioMode; }
    int		 mode()	  const { return ioMode & IO_ModeMask; }
    int		 state()  const { return ioMode & IO_StateMask; }

    bool	 isDirectAccess()     const { return ((ioMode & IO_Direct)     == IO_Direct); }
    bool	 isSequentialAccess() const { return ((ioMode & IO_Sequential) == IO_Sequential); }
    bool	 isCombinedAccess()   const { return ((ioMode & IO_Combined)   == IO_Combined); }
    bool	 isBuffered()	      const { return ((ioMode & IO_Raw)        != IO_Raw); }
    bool	 isRaw()	      const { return ((ioMode & IO_Raw)        == IO_Raw); }
    bool	 isSynchronous()      const { return ((ioMode & IO_Async)      != IO_Async); }
    bool	 isAsynchronous()     const { return ((ioMode & IO_Async)      == IO_Async); }
    bool	 isTranslated()	      const { return ((ioMode & IO_Translate)  == IO_Translate); }
    bool	 isReadable()	      const { return ((ioMode & IO_ReadOnly)   == IO_ReadOnly); }
    bool	 isWritable()	      const { return ((ioMode & IO_WriteOnly)  == IO_WriteOnly); }
    bool	 isReadWrite()	      const { return ((ioMode & IO_ReadWrite)  == IO_ReadWrite); }
    bool	 isInactive()	      const { return state() == 0; }
    bool	 isOpen()	      const { return state() == IO_Open; }

    int		 status() const { return ioSt; }
    void	 resetqStatus()	{ ioSt = IO_Ok; }

    virtual bool open( int mode ) = 0;
    virtual void close() = 0;
    virtual void flush() = 0;

    virtual Offset size()  const = 0;
    virtual Offset at()  const;
    virtual bool at( Offset );
    virtual bool atEnd()  const;
    bool	 reset() { return at(0); }

    virtual TQ_LONG readBlock( char *data, TQ_ULONG maxlen ) = 0;
    virtual TQ_LONG writeBlock( const char *data, TQ_ULONG len ) = 0;
    virtual TQ_LONG readLine( char *data, TQ_ULONG maxlen );
    TQ_LONG writeBlock( const TQByteArray& data );
    virtual TQByteArray readAll();

    virtual int	 getch() = 0;
    virtual int	 putch( int ) = 0;
    virtual int	 ungetch( int ) = 0;

protected:
    void	 setFlags( int f ) { ioMode = f; }
    void	 setType( int );
    void	 setMode( int );
    void	 setState( int );
    void	 setqStatus( int );
    Offset	 ioIndex;

private:
    int		 ioMode;
    int		 ioSt;

private:	// Disabled copy constructor and operator=
#if defined(TQ_DISABLE_COPY)
    TQIODevice( const TQIODevice & );
    TQIODevice &operator=( const TQIODevice & );
#endif
};

#endif // USE_QT4

#endif // TQIODEVICE_H
