/****************************************************************************
**
** Definition of TQMainWindow class
**
** Created : 980316
**
** Copyright (C) 2010 Timothy Pearson and (C) 1992-2008 Trolltech ASA.
**
** This file is part of the widgets module of the TQt GUI Toolkit.
**
** This file may be used under the terms of the GNU General
** Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the files LICENSE.GPL2
** and LICENSE.GPL3 included in the packaging of this file.
** Alternatively you may (at your option) use any later version
** of the GNU General Public License if such license has been
** publicly approved by Trolltech ASA (or its successors, if any)
** and the KDE Free TQt Foundation.
**
** Please review the following information to ensure GNU General
** Public Licensing requirements will be met:
** http://trolltech.com/products/qt/licenses/licensing/opensource/.
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
** or contact the sales department at sales@trolltech.com.
**
** This file may be used under the terms of the Q Public License as
** defined by Trolltech ASA and appearing in the file LICENSE.TQPL
** included in the packaging of this file.  Licensees holding valid TQt
** Commercial licenses may use this file in accordance with the TQt
** Commercial License Agreement provided with the Software.
**
** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
** herein.
**
**********************************************************************/

#ifndef TQMAINWINDOW_H
#define TQMAINWINDOW_H

#ifndef TQT_H
#include "tqwidget.h"
#include "tqtoolbar.h"
#include "tqptrlist.h"
#include "tqtextstream.h"
#endif // TQT_H

#ifndef TQT_NO_MAINWINDOW

class TQMenuBar;
class TQStatusBar;
class TQToolTipGroup;
class TQMainWindowPrivate;
class TQMainWindowLayout;
class TQPopupMenu;

class TQ_EXPORT TQMainWindow: public TQWidget
{
    Q_OBJECT
    TQ_OBJECT
    Q_PROPERTY( bool rightJustification READ rightJustification WRITE setRightJustification DESIGNABLE false )
    Q_PROPERTY( bool usesBigPixmaps READ usesBigPixmaps WRITE setUsesBigPixmaps )
    Q_PROPERTY( bool usesTextLabel READ usesTextLabel WRITE setUsesTextLabel )
    Q_PROPERTY( bool dockWindowsMovable READ dockWindowsMovable WRITE setDockWindowsMovable )
    Q_PROPERTY( bool opaqueMoving READ opaqueMoving WRITE setOpaqueMoving )

public:
    TQMainWindow( TQWidget* parent=0, const char* name=0, WFlags f = (WFlags)TQt::WType_TopLevel );
    ~TQMainWindow();

#ifndef TQT_NO_MENUBAR
    TQMenuBar * menuBar() const;
#endif
    TQStatusBar * statusBar() const;
#ifndef TQT_NO_TOOLTIP
    TQToolTipGroup * toolTipGroup() const;
#endif

    virtual void setCentralWidget( TQWidget * );
    TQWidget * centralWidget() const;

    virtual void setDockEnabled( TQt::Dock dock, bool enable );
    bool isDockEnabled( TQt::Dock dock ) const;
    bool isDockEnabled( TQDockArea *area ) const;
    virtual void setDockEnabled( TQDockWindow *tb, TQt::Dock dock, bool enable );
    bool isDockEnabled( TQDockWindow *tb, TQt::Dock dock ) const;
    bool isDockEnabled( TQDockWindow *tb, TQDockArea *area ) const;

    virtual void addDockWindow( TQDockWindow *, TQt::Dock = TQt::DockTop, bool newLine = FALSE );
    virtual void addDockWindow( TQDockWindow *, const TQString &label,
				TQt::Dock = TQt::DockTop, bool newLine = FALSE );
    virtual void moveDockWindow( TQDockWindow *, TQt::Dock = TQt::DockTop );
    virtual void moveDockWindow( TQDockWindow *, TQt::Dock, bool nl, int index, int extraOffset = -1 );
    virtual void removeDockWindow( TQDockWindow * );

    void show();
    void hide();
    TQSize tqsizeHint() const;
    TQSize tqminimumSizeHint() const;

    bool rightJustification() const;
    bool usesBigPixmaps() const;
    bool usesTextLabel() const;
    bool dockWindowsMovable() const;
    bool opaqueMoving() const;

    bool eventFilter( TQObject*, TQEvent* );

    bool getLocation( TQDockWindow *tb, TQt::Dock &dock, int &index, bool &nl, int &extraOffset ) const;

    TQPtrList<TQDockWindow> dockWindows( TQt::Dock dock ) const;
    TQPtrList<TQDockWindow> dockWindows() const;
    void lineUpDockWindows( bool keepNewLines = FALSE );

    bool isDockMenuEnabled() const;

    // compatibility stuff
    bool hasDockWindow( TQDockWindow *dw );
#ifndef TQT_NO_TOOLBAR
    void addToolBar( TQDockWindow *, TQt::Dock = TQt::DockTop, bool newLine = FALSE );
    void addToolBar( TQDockWindow *, const TQString &label,
		     TQt::Dock = TQt::DockTop, bool newLine = FALSE );
    void moveToolBar( TQDockWindow *, TQt::Dock = TQt::DockTop );
    void moveToolBar( TQDockWindow *, TQt::Dock, bool nl, int index, int extraOffset = -1 );
    void removeToolBar( TQDockWindow * );

    bool toolBarsMovable() const;
    TQPtrList<TQToolBar> toolBars( TQt::Dock dock ) const;
    void lineUpToolBars( bool keepNewLines = FALSE );
#endif

    virtual TQDockArea *dockingArea( const TQPoint &p );
    TQDockArea *leftDock() const;
    TQDockArea *rightDock() const;
    TQDockArea *topDock() const;
    TQDockArea *bottomDock() const;

    virtual bool isCustomizable() const;

    bool appropriate( TQDockWindow *dw ) const;

    enum DockWindows { OnlyToolBars, NoToolBars, AllDockWindows };
    TQPopupMenu *createDockWindowMenu( DockWindows dockWindows = AllDockWindows ) const;

public Q_SLOTS:
    virtual void setRightJustification( bool );
    virtual void setUsesBigPixmaps( bool );
    virtual void setUsesTextLabel( bool );
    virtual void setDockWindowsMovable( bool );
    virtual void setOpaqueMoving( bool );
    virtual void setDockMenuEnabled( bool );
    virtual void whatsThis();
    virtual void setAppropriate( TQDockWindow *dw, bool a );
    virtual void customize();

    // compatibility stuff
    void setToolBarsMovable( bool );

Q_SIGNALS:
    void pixmapSizeChanged( bool );
    void usesTextLabelChanged( bool );
    void dockWindowPositionChanged( TQDockWindow * );

#ifndef TQT_NO_TOOLBAR
    // compatibility stuff
    void toolBarPositionChanged( TQToolBar * );
#endif

protected Q_SLOTS:
    virtual void setUpLayout();
    virtual bool showDockMenu( const TQPoint &globalPos );
    void menuAboutToShow();

protected:
    void paintEvent( TQPaintEvent * );
    void childEvent( TQChildEvent * );
    bool event( TQEvent * );
    void styleChange( TQStyle& );

private Q_SLOTS:
    void slotPlaceChanged();
    void doLineUp() { lineUpDockWindows( TRUE ); }

private:
    TQMainWindowPrivate * d;
    void triggerLayout( bool deleteLayout = TRUE);
    bool dockMainWindow( TQObject *dock );

#ifndef TQT_NO_MENUBAR
    virtual void setMenuBar( TQMenuBar * );
#endif
    virtual void setqStatusBar( TQStatusBar * );
#ifndef TQT_NO_TOOLTIP
    virtual void setToolTipGroup( TQToolTipGroup * );
#endif

    friend class TQDockWindow;
    friend class TQMenuBar;
    friend class TQHideDock;
    friend class TQToolBar;
    friend class TQMainWindowLayout;
private:	// Disabled copy constructor and operator=
#if defined(TQ_DISABLE_COPY)
    TQMainWindow( const TQMainWindow & );
    TQMainWindow& operator=( const TQMainWindow & );
#endif
};

#ifndef TQT_NO_TOOLBAR
inline void TQMainWindow::addToolBar( TQDockWindow *w, TQt::ToolBarDock dock, bool newLine )
{
    addDockWindow( w, dock, newLine );
}

inline void TQMainWindow::addToolBar( TQDockWindow *w, const TQString &label,
			      TQt::ToolBarDock dock, bool newLine )
{
    addDockWindow( w, label, dock, newLine );
}

inline void TQMainWindow::moveToolBar( TQDockWindow *w, TQt::ToolBarDock dock )
{
    moveDockWindow( w, dock );
}

inline void TQMainWindow::moveToolBar( TQDockWindow *w, TQt::ToolBarDock dock, bool nl, int index, int extraOffset )
{
    moveDockWindow( w, dock, nl, index, extraOffset );
}

inline void TQMainWindow::removeToolBar( TQDockWindow *w )
{
    removeDockWindow( w );
}

inline bool TQMainWindow::toolBarsMovable() const
{
    return dockWindowsMovable();
}

inline void TQMainWindow::lineUpToolBars( bool keepNewLines )
{
    lineUpDockWindows( keepNewLines );
}

inline void TQMainWindow::setToolBarsMovable( bool b )
{
    setDockWindowsMovable( b );
}
#endif

#ifndef TQT_NO_TEXTSTREAM
TQ_EXPORT TQTextStream &operator<<( TQTextStream &, const TQMainWindow & );
TQ_EXPORT TQTextStream &operator>>( TQTextStream &, TQMainWindow & );
#endif

#endif // TQT_NO_MAINWINDOW

#endif // TQMAINWINDOW_H
