#include "tqtglobaldefines.h"

#ifdef USE_QT4

/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the Qt3Support module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef TQWIDGETSTACK_H
#define TQWIDGETSTACK_H

#include <tqframe.h>
#include <tqintdict.h>
#include <tqptrdict.h>

QT_BEGIN_HEADER

QT_BEGIN_NAMESPACE

QT_MODULE(Qt3SupportLight)

class TQWidgetStackPrivate;


class Q_COMPAT_EXPORT TQWidgetStack: public TQFrame
{
    Q_OBJECT
    TQ_OBJECT
public:
    TQWidgetStack(TQWidget* parent, const char* name=0, WFlags f=0);

    ~TQWidgetStack();

    int addWidget(TQWidget *, int = -1);
    void removeWidget(TQWidget *);

    TQSize tqsizeHint() const;
    TQSize tqminimumSizeHint() const;
    void setVisible(bool visible);

    TQWidget * widget(int) const;
    int id(TQWidget *) const;

    TQWidget * visibleWidget() const;

    void setFrameRect(const QRect &);

Q_SIGNALS:
    void aboutToShow(int);
    void aboutToShow(TQWidget *);

public Q_SLOTS:
    void raiseWidget(int);
    void raiseWidget(TQWidget *);

protected:
    void frameChanged();
    void resizeEvent(TQResizeEvent *);
    bool event(TQEvent* e);

    virtual void setChildGeometries();
    void childEvent(TQChildEvent *);

private:
    void init();

    TQWidgetStackPrivate * d;
    TQIntDict<TQWidget> * dict;
    TQPtrDict<TQWidget> * focusWidgets;
    TQWidget * topWidget;
    TQWidget * invisible;

    Q_DISABLE_COPY(TQWidgetStack)
};

QT_END_NAMESPACE

QT_END_HEADER

#endif // TQWIDGETSTACK_H

#else // USE_QT4

/****************************************************************************
**
** Definition of TQWidgetStack class
**
** Created : 980306
**
** Copyright (C) 2010 Timothy Pearson and (C) 1992-2008 Trolltech ASA.
**
** This file is part of the widgets module of the TQt GUI Toolkit.
**
** This file may be used under the terms of the GNU General
** Public License versions 2.0 or 3.0 as published by the Free
** Software Foundation and appearing in the files LICENSE.GPL2
** and LICENSE.GPL3 included in the packaging of this file.
** Alternatively you may (at your option) use any later version
** of the GNU General Public License if such license has been
** publicly approved by Trolltech ASA (or its successors, if any)
** and the KDE Free TQt Foundation.
**
** Please review the following information to ensure GNU General
** Public Licensing requirements will be met:
** http://trolltech.com/products/qt/licenses/licensing/opensource/.
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
** or contact the sales department at sales@trolltech.com.
**
** This file may be used under the terms of the Q Public License as
** defined by Trolltech ASA and appearing in the file LICENSE.TQPL
** included in the packaging of this file.  Licensees holding valid TQt
** Commercial licenses may use this file in accordance with the TQt
** Commercial License Agreement provided with the Software.
**
** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
** herein.
**
**********************************************************************/

#ifndef TQWIDGETSTACK_H
#define TQWIDGETSTACK_H

#ifndef TQT_H
#include "tqframe.h"
#include "tqintdict.h"
#include "tqptrdict.h"
#endif // TQT_H

#ifndef TQT_NO_WIDGETSTACK


class TQWidgetStackPrivate;


class TQ_EXPORT TQWidgetStack: public TQFrame
{
    Q_OBJECT
    TQ_OBJECT
public:
    TQWidgetStack( TQWidget* parent=0, const char* name=0 );
    TQWidgetStack( TQWidget* parent, const char* name, WFlags f);

    ~TQWidgetStack();

    int addWidget( TQWidget *, int = -1 );
    void removeWidget( TQWidget * );

    TQSize tqsizeHint() const;
    TQSize tqminimumSizeHint() const;
    void show();

    TQWidget * widget( int ) const;
    int id( TQWidget * ) const;

    TQWidget * visibleWidget() const;

    void setFrameRect( const TQRect & );

Q_SIGNALS:
    void aboutToShow( int );
    void aboutToShow( TQWidget * );

public Q_SLOTS:
    void raiseWidget( int );
    void raiseWidget( TQWidget * );

protected:
    void frameChanged();
    void resizeEvent( TQResizeEvent * );

    virtual void setChildGeometries();
    void childEvent( TQChildEvent * );

private:
    void init();

    TQWidgetStackPrivate * d;
    TQIntDict<TQWidget> * dict;
    TQPtrDict<TQWidget> * tqfocusWidgets;
    TQWidget * topWidget;
    TQWidget * invisible;

private:	// Disabled copy constructor and operator=
#if defined(TQ_DISABLE_COPY)
    TQWidgetStack( const TQWidgetStack & );
    TQWidgetStack& operator=( const TQWidgetStack & );
#endif
};

#endif // TQT_NO_WIDGETSTACK

#endif // TQWIDGETSTACK_H

#endif // USE_QT4
